/*
   MyideDiskUnix - direct drive access for Unix

   Copyright (C) 2006 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include "MyideDiskUnix.h"
#include "Error.h"

MyideDiskUnix::MyideDiskUnix(const char* filename)
	: super()
{
	fFile = fopen64(filename, "r+");
	if (!fFile) {
		throw FileOpenError(filename);
	}
	if (!ReadPartitionTable()) {
		throw ErrorObject("Reading partition table failed");
	}
}

MyideDiskUnix::~MyideDiskUnix()
{
	if (fFile) {
		fclose(fFile);
		fFile = NULL;
	}
}

bool MyideDiskUnix::SeekToSector(unsigned int sector)
{
	if (!fFile) {
		return false;
	}
	off64_t pos = (off64_t) sector * 512;
	if (fseeko64(fFile, pos, SEEK_SET)) {
		DPRINTF("seeking to sector %d failed\n", sector);
		return false;
	}
	return true;
}

bool MyideDiskUnix::ReadBlock(unsigned int sector)
{
	if (!SeekToSector(sector)) {
		return false;
	}
	if (fread(fSectorBuffer, 512, 1, fFile) != 1) {
		DPRINTF("reading sector %d failed\n", sector);
		return false;
	}
	return true;
}

bool MyideDiskUnix::WriteBlock(unsigned int sector)
{
	if (!SeekToSector(sector)) {
		return false;
	}
	if (fwrite(fSectorBuffer, 512, 1, fFile) != 1) {
		DPRINTF("writing sector %d failed\n", sector);
		return false;
	}
	return true;
}

