/*
   MyideDiskWin32 - direct drive access for Win32

   Copyright (C) 2006 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "MyideDiskWin32.h"
#include "Error.h"
#include "AtariDebug.h"

MyideDiskWin32::MyideDiskWin32(const char* filename)
	: super()
{
	fHandle = CreateFile(filename, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE , NULL, OPEN_EXISTING, 0, 0);
	if (fHandle == INVALID_HANDLE_VALUE) {
		throw FileOpenError(filename);
	}
	if (!ReadPartitionTable()) {
		ErrorObject("Reading partition table failed");
	}
}

MyideDiskWin32::~MyideDiskWin32()
{
	if (fHandle != INVALID_HANDLE_VALUE) {
		CloseHandle(fHandle);
		fHandle = INVALID_HANDLE_VALUE;
	}
}

bool MyideDiskWin32::SeekToSector(unsigned int sector)
{
	LONGLONG pos = (unsigned long long) sector * 512;
	LONG lo = pos & 0xffffffff;
	LONG hi = pos >> 32;

	if (SetFilePointer(fHandle, lo, &hi, FILE_BEGIN) != (DWORD)lo) {
		DPRINTF("seeking to sector %d failed\n", sector);
		return false;
	}
	return true;
}

bool MyideDiskWin32::ReadBlock(unsigned int sector)
{
	if (!SeekToSector(sector)) {
		return false;
	}
	DWORD len;
	if (!ReadFile(fHandle, fSectorBuffer, 512, &len, NULL)) {
		DPRINTF("reading sector %d failed\n", sector);
		return false;
	}
	return true;
}

bool MyideDiskWin32::WriteBlock(unsigned int sector)
{
	if (!SeekToSector(sector)) {
		return false;
	}
	DWORD len;
	if (!WriteFile(fHandle, fSectorBuffer, 512, &len, NULL)) {
		DPRINTF("writing sector %d failed\n", sector);
		return false;
	}
	return true;
}
